@@ -59,7 +59,10 @@
 #define GSD_XRANDR_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_XRANDR_MANAGER, GsdXrandrManagerPrivate))
 
 #define CONF_DIR "/apps/gnome_settings_daemon/xrandr"
-#define CONF_KEY "show_notification_icon"
+#define CONF_KEY_SHOW_NOTIFICATION_ICON (CONF_DIR "/show_notification_icon")
+#define CONF_KEY_TURN_ON_EXTERNAL_MONITORS_AT_STARTUP	(CONF_DIR "/turn_on_external_monitors_at_startup")
+#define CONF_KEY_TURN_ON_LAPTOP_MONITOR_AT_STARTUP	(CONF_DIR "/turn_on_laptop_monitor_at_startup")
+#define CONF_KEY_DEFAULT_CONFIGURATION_FILE             (CONF_DIR "/default_configuration_file")
 
 #define VIDEO_KEYSYM    "XF86Display"
 #define ROTATE_KEYSYM   "XF86RotateWindows"
@@ -988,8 +991,6 @@ handle_fn_f7 (GsdXrandrManager *mgr, guint32 timestamp)
         gnome_rr_config_free (current);
 
         if (priv->fn_f7_configs) {
-                guint32 server_timestamp;
-
                 mgr->priv->current_fn_f7_config++;
 
                 if (priv->fn_f7_configs[mgr->priv->current_fn_f7_config] == NULL)
@@ -1001,22 +1002,6 @@ handle_fn_f7 (GsdXrandrManager *mgr, guint32 timestamp)
 
                 g_debug ("applying");
 
-                /* See https://bugzilla.gnome.org/show_bug.cgi?id=610482
-                 *
-                 * Sometimes we'll get two rapid XF86Display keypress events,
-                 * but their timestamps will be out of order with respect to the
-                 * RANDR timestamps.  This *may* be due to stupid BIOSes sending
-                 * out display-switch keystrokes "to make Windows work".
-                 *
-                 * The X server will error out if the timestamp provided is
-                 * older than a previous change configuration timestamp. We
-                 * assume here that we do want this event to go through still,
-                 * since kernel timestamps may be skewed wrt the X server.
-                 */
-                gnome_rr_screen_get_timestamps (screen, NULL, &server_timestamp);
-                if (timestamp < server_timestamp)
-                        timestamp = server_timestamp;
-
                 apply_configuration_and_display_error (mgr, priv->fn_f7_configs[mgr->priv->current_fn_f7_config], timestamp);
         }
         else {
@@ -1499,6 +1484,7 @@ output_title_label_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gp
         GnomeOutputInfo *output;
         GdkColor color;
         cairo_t *cr;
+        GtkAllocation allocation;
 
         g_assert (GTK_IS_LABEL (widget));
 
@@ -1511,23 +1497,24 @@ output_title_label_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gp
 
         gnome_rr_labeler_get_color_for_output (priv->labeler, output, &color);
 
-        cr = gdk_cairo_create (widget->window);
+        cr = gdk_cairo_create (gtk_widget_get_window (widget));
 
         cairo_set_source_rgb (cr, 0, 0, 0);
         cairo_set_line_width (cr, OUTPUT_TITLE_ITEM_BORDER);
+        gtk_widget_get_allocation (widget, &allocation);
         cairo_rectangle (cr,
-                         widget->allocation.x + OUTPUT_TITLE_ITEM_BORDER / 2.0,
-                         widget->allocation.y + OUTPUT_TITLE_ITEM_BORDER / 2.0,
-                         widget->allocation.width - OUTPUT_TITLE_ITEM_BORDER,
-                         widget->allocation.height - OUTPUT_TITLE_ITEM_BORDER);
+                         allocation.x + OUTPUT_TITLE_ITEM_BORDER / 2.0,
+                         allocation.y + OUTPUT_TITLE_ITEM_BORDER / 2.0,
+                         allocation.width - OUTPUT_TITLE_ITEM_BORDER,
+                         allocation.height - OUTPUT_TITLE_ITEM_BORDER);
         cairo_stroke (cr);
 
         gdk_cairo_set_source_color (cr, &color);
         cairo_rectangle (cr,
-                         widget->allocation.x + OUTPUT_TITLE_ITEM_BORDER,
-                         widget->allocation.y + OUTPUT_TITLE_ITEM_BORDER,
-                         widget->allocation.width - 2 * OUTPUT_TITLE_ITEM_BORDER,
-                         widget->allocation.height - 2 * OUTPUT_TITLE_ITEM_BORDER);
+                         allocation.x + OUTPUT_TITLE_ITEM_BORDER,
+                         allocation.y + OUTPUT_TITLE_ITEM_BORDER,
+                         allocation.width - 2 * OUTPUT_TITLE_ITEM_BORDER,
+                         allocation.height - 2 * OUTPUT_TITLE_ITEM_BORDER);
 
         cairo_fill (cr);
 
@@ -1542,7 +1529,7 @@ output_title_label_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gp
          * Yay for fucking with GTK+'s internals.
          */
 
-        widget->state = GTK_STATE_NORMAL;
+        gtk_widget_set_state (widget, GTK_STATE_NORMAL);
 
         return FALSE;
 }
@@ -1552,7 +1539,7 @@ static gboolean
 output_title_label_after_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 {
         g_assert (GTK_IS_LABEL (widget));
-        widget->state = GTK_STATE_INSENSITIVE;
+        gtk_widget_set_state (widget, GTK_STATE_INSENSITIVE);
 
         return FALSE;
 }
@@ -1618,7 +1605,7 @@ make_menu_item_for_output_title (GsdXrandrManager *manager, GnomeOutputInfo *out
 	 * theme's colors, since the label is always shown against a light
 	 * pastel background.  See bgo#556050
 	 */
-	gtk_widget_modify_fg (label, GTK_WIDGET_STATE (label), &black);
+	gtk_widget_modify_fg (label, gtk_widget_get_state (label), &black);
 
         /* Add padding around the label to fit the box that we'll draw for color-coding */
         gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -1690,11 +1677,14 @@ add_unsupported_rotation_item (GsdXrandrManager *manager)
         struct GsdXrandrManagerPrivate *priv = manager->priv;
         GtkWidget *item;
         GtkWidget *label;
+        gchar *markup;
 
         item = gtk_menu_item_new ();
 
         label = gtk_label_new (NULL);
-        gtk_label_set_markup (GTK_LABEL (label), _("<i>Rotation not supported</i>"));
+        markup = g_strdup_printf ("<i>%s</i>", _("Rotation not supported"));
+        gtk_label_set_markup (GTK_LABEL (label), markup);
+        g_free (markup);
         gtk_container_add (GTK_CONTAINER (item), label);
 
         gtk_widget_show_all (item);
@@ -1889,7 +1879,7 @@ status_icon_popup_menu (GsdXrandrManager *manager, guint button, guint32 timesta
         gtk_widget_show (item);
         gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
 
-        item = gtk_menu_item_new_with_mnemonic (_("_Configure Display Settings ..."));
+        item = gtk_menu_item_new_with_mnemonic (_("_Configure Display Settings…"));
         g_signal_connect (item, "activate",
                           G_CALLBACK (popup_menu_configure_display_cb), manager);
         gtk_widget_show (item);
@@ -1961,7 +1951,7 @@ status_icon_stop (GsdXrandrManager *manager)
 static void
 start_or_stop_icon (GsdXrandrManager *manager)
 {
-        if (gconf_client_get_bool (manager->priv->client, CONF_DIR "/" CONF_KEY, NULL)) {
+        if (gconf_client_get_bool (manager->priv->client, CONF_KEY_SHOW_NOTIFICATION_ICON, NULL)) {
                 status_icon_start (manager);
         }
         else {
@@ -1975,16 +1965,19 @@ on_config_changed (GConfClient          *client,
                    GConfEntry           *entry,
                    GsdXrandrManager *manager)
 {
-        start_or_stop_icon (manager);
+        if (strcmp (entry->key, CONF_KEY_SHOW_NOTIFICATION_ICON) == 0)
+                start_or_stop_icon (manager);
 }
 
-static void
+static gboolean
 apply_intended_configuration (GsdXrandrManager *manager, const char *intended_filename, guint32 timestamp)
 {
         GError *my_error;
+	gboolean result;
 
         my_error = NULL;
-        if (!apply_configuration_from_filename (manager, intended_filename, FALSE, timestamp, &my_error)) {
+        result = apply_configuration_from_filename (manager, intended_filename, FALSE, timestamp, &my_error);
+        if (!result) {
                 if (my_error) {
                         if (!g_error_matches (my_error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
                                 error_message (manager, _("Could not apply the stored configuration for monitors"), my_error, NULL);
@@ -1992,9 +1985,39 @@ apply_intended_configuration (GsdXrandrManager *manager, const char *intended_fi
                         g_error_free (my_error);
                 }
         }
+
+        return result;
 }
 
 static void
+apply_default_boot_configuration (GsdXrandrManager *mgr, guint32 timestamp)
+{
+	GsdXrandrManagerPrivate *priv = mgr->priv;
+	GnomeRRScreen *screen = priv->rw_screen;
+   	GnomeRRConfig *config;
+   	gboolean turn_on_external, turn_on_laptop;
+
+        turn_on_external =
+                gconf_client_get_bool (mgr->priv->client, CONF_KEY_TURN_ON_EXTERNAL_MONITORS_AT_STARTUP, NULL);
+        turn_on_laptop =
+                gconf_client_get_bool (mgr->priv->client, CONF_KEY_TURN_ON_LAPTOP_MONITOR_AT_STARTUP, NULL);
+
+	if (turn_on_external && turn_on_laptop)
+		config = make_clone_setup (screen);
+	else if (!turn_on_external && turn_on_laptop)
+		config = make_laptop_setup (screen);
+	else if (turn_on_external && !turn_on_laptop)
+		config = make_other_setup (screen);
+	else
+		config = make_laptop_setup (screen);
+
+        if (config) {
+                apply_configuration_and_display_error (mgr, config, timestamp);
+                gnome_rr_config_free (config);
+        }
+}
+
+static gboolean
 apply_stored_configuration_at_startup (GsdXrandrManager *manager, guint32 timestamp)
 {
         GError *my_error;
@@ -2038,7 +2061,7 @@ apply_stored_configuration_at_startup (GsdXrandrManager *manager, guint32 timest
          * good.  Apply the intended configuration instead.
          */
 
-        apply_intended_configuration (manager, intended_filename, timestamp);
+        success = apply_intended_configuration (manager, intended_filename, timestamp);
 
 out:
 
@@ -2047,6 +2070,25 @@ out:
 
         g_free (backup_filename);
         g_free (intended_filename);
+
+        return success;
+}
+
+static gboolean
+apply_default_configuration_from_file (GsdXrandrManager *manager, guint32 timestamp)
+{
+        GsdXrandrManagerPrivate *priv = manager->priv;
+        char *default_config_filename;
+        gboolean result;
+
+        default_config_filename = gconf_client_get_string (priv->client, CONF_KEY_DEFAULT_CONFIGURATION_FILE, NULL);
+        if (!default_config_filename)
+                return FALSE;
+
+        result = apply_configuration_from_filename (manager, default_config_filename, TRUE, timestamp, NULL);
+
+        g_free (default_config_filename);
+        return result;
 }
 
 gboolean
@@ -2102,7 +2144,9 @@ gsd_xrandr_manager_start (GsdXrandrManager *manager,
         }
 
         show_timestamps_dialog (manager, "Startup");
-        apply_stored_configuration_at_startup (manager, GDK_CURRENT_TIME); /* we don't have a real timestamp at startup anyway */
+        if (!apply_stored_configuration_at_startup (manager, GDK_CURRENT_TIME)) /* we don't have a real timestamp at startup anyway */
+                if (!apply_default_configuration_from_file (manager, GDK_CURRENT_TIME))
+                        apply_default_boot_configuration (manager, GDK_CURRENT_TIME);
 
         gdk_window_add_filter (gdk_get_default_root_window(),
                                (GdkFilterFunc)event_filter,
